@@ -1,7 +1,7 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/get.c,v 1.42 2005/02/07 16:38:58 richb Exp $
+/*  $Header$
  *
- *  Copyright (c) 1987-2005 Sun Microsystems, Inc. All Rights Reserved.
+ *  Copyright (c) 1987-2007 Sun Microsystems, Inc. All Rights Reserved.
  *           
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,14 +29,64 @@
 #include <sys/types.h>
 #include <sys/file.h>
 #include <sys/param.h>
-#include "calctool.h"
-#include "extern.h"
+#include <assert.h>
+#include <gconf/gconf-client.h>
 
-static int get_bool_resource(enum res_type, int *);
-static int get_int_resource(enum res_type, int *);
-static int get_str_resource(enum res_type, char *);
+#include "get.h"
+#include "display.h"
+#include "mp.h"
 
-static void getparam(char *, char **, char *);
+#define EQUAL(a, b)    (strlen(a)==strlen(b)) & !strcmp(a, b) 
+
+/* Various string values read/written as X resources. */
+
+char *Rbstr[MAXBASES]     = { "BIN", "OCT", "DEC", "HEX" };
+char *Rdstr[MAXDISPMODES] = { "ENG", "FIX", "SCI" };
+char *Rmstr[MAXMODES]     = { "BASIC", "ADVANCED", "FINANCIAL", 
+                              "SCIENTIFIC" };
+char *Rtstr[MAXTRIGMODES] = { "DEG", "GRAD", "RAD" };
+char *Rsstr[MAXSYNTAX]    = { "ARITHMETIC", "ARITHMETIC_PRECEDENCE" };
+char *Rcstr[MAXBITCALC]   = { "NO_BITCALCULATING_MODE", "BITCALCULATING_MODE" };
+
+static GConfClient *client = NULL;
+
+
+char *
+get_resource(char *key)
+{
+    char key_name[MAXLINE];
+    SNPRINTF(key_name, MAXLINE, "/apps/%s/%s", v->appname, key);
+    return(gconf_client_get_string(client, key_name, NULL));
+}
+
+
+void
+set_resource(char *key, char *value)
+{
+    char key_name[MAXLINE];
+    SNPRINTF(key_name, MAXLINE, "/apps/%s/%s", v->appname, key);    
+    gconf_client_set_string(client, key_name, value, NULL);
+}
+
+
+void
+set_int_resource(char *key, int value)
+{
+    char intvalue[MAXLINE];
+    SNPRINTF(intvalue, MAXLINE, "%d", value);
+    set_resource(key, intvalue);
+}
+
+
+void
+set_boolean_resource(char *key, int value)
+{
+    if (value) {
+        set_resource(key, "true");
+    } else {
+        set_resource(key, "false");
+    }
+}
 
 
 char *
@@ -61,19 +111,17 @@ convert(char *line)       /* Convert .gcalctoolcf line to ascii values. */
 }
 
 
-/* Get boolean resource from database. */
-
-static int
-get_bool_resource(enum res_type rtype, int *boolval)
+int
+get_boolean_resource(char *key, int *boolval)
 {
     char *val, tempstr[MAXLINE];
     int len, n;
 
-    if ((val = get_resource(rtype)) == NULL) {
+    if ((val = get_resource(key)) == NULL) {
         g_free(val);
         return(0);
     }
-    STRCPY(tempstr, val);
+    STRNCPY(tempstr, val, MAXLINE - 1);
     g_free(val);
     len = strlen(tempstr);
     for (n = 0; n < len; n++) {
@@ -93,12 +141,12 @@ get_bool_resource(enum res_type rtype, int *boolval)
 
 /* Get integer resource from database. */
 
-static int
-get_int_resource(enum res_type rtype, int *intval)
+int
+get_int_resource(char *key, int *intval)
 {
     char *val;
  
-    if ((val = get_resource(rtype)) == NULL) {
+    if ((val = get_resource(key)) == NULL) {
         g_free(val);
         return(0);
     }
@@ -131,73 +179,19 @@ get_radix()
 }
 
 
-void
-get_options(int argc, char *argv[])      /* Extract command line options. */
-{
-    char next[MAXLINE];       /* The next command line parameter. */
-
-    INC;
-    while (argc > 0) {
-        if (argv[0][0] == '-') {
-            switch (argv[0][1]) {
-                case 'D' :                   /* MP debug info. to stderr. */
-                    v->MPdebug = TRUE;
-                    break;
-
-                case 'E' :                   /* MP errors to stderr. */
-                    v->MPerrors = TRUE;
-                    break;
-
-                case 'a' : 
-                    INC;
-                    getparam(next, argv, _("-a needs accuracy value"));
-                    v->accuracy = atoi(next);
-                    if (v->accuracy < 0 || v->accuracy > MAXACC) {
-                        FPRINTF(stderr, 
-                                _("%s: accuracy should be in the range 0-%d\n"),
-                                v->progname, MAXACC);
-                        v->accuracy = 9;
-                    }
-                    break;
-
-                case '?' :
-                case 'v' : 
-                    usage(v->progname);
-                    break;
-            }
-            INC;
-        } else {
-            INC;
-        }
-    }
-}
-
-
-static void
-getparam(char *s, char *argv[], char *errmes)
-{
-    if (*argv != NULL && argv[0][0] != '-') {
-        STRCPY(s, *argv);
-    } else { 
-        FPRINTF(stderr, _("%s: %s as next argument.\n"), v->progname, errmes);
-        exit(1);                        
-    }                                  
-}
-
-
 /* Get a string resource from database. */
 
 static int
-get_str_resource(enum res_type rtype, char *strval)
+get_str_resource(char *key, char *strval)
 {
     char *val;
     int i, len;
 
-    if ((val = get_resource(rtype)) == NULL) {
+    if ((val = get_resource(key)) == NULL) {
         g_free(val);
         return(0);
     }
-    STRCPY(strval, val);
+    STRNCPY(strval, val, MAXLINE - 1);
     g_free(val);
     len = strlen(strval);
     for (i = 0; i < len; i++) {
@@ -232,78 +226,11 @@ get_tsep()
 }
 
 
-static void
-init_constant(int n, gchar *value)
-{
-    gchar *str = g_strdup(value);
-
-    MPstr_to_num(str, DEC, v->MPcon_vals[n]);
-    g_free(str);
-}
-
-
-void
-init_vars()    /* Setup default values for various variables. */
-{
-    int acc, i, n, size;
-
-    v->accuracy      = 9;      /* Initial accuracy. */
-    v->show_zeroes   = FALSE;  /* Don't show trailing zeroes. */
-    v->base          = DEC;    /* Initial base. */
-    v->dtype         = FIX;    /* Initial number display mode. */
-    v->ttype         = DEG;    /* Initial trigonometric type. */
-    v->modetype      = BASIC;  /* Initial calculator mode. */
-    v->rstate        = 0;      /* No memory register frame display initially. */
-    v->iconic        = FALSE;  /* Calctool not iconic by default. */
-    v->MPdebug       = FALSE;  /* No debug info by default. */
-    v->MPerrors      = FALSE;               /* No error information. */
-    acc              = MAX_DIGITS + 12;     /* MP internal accuracy. */
-    size             = MP_SIZE;
-    mpset(&acc, &size, &size);
-
-    v->error       = 0;            /* No calculator error initially. */
-    v->key_exp     = 0;            /* Not entering an exponent number. */
-    v->pending_op  = 0;            /* No pending arithmetic operation. */
-    v->titleline   = NULL;         /* No User supplied title line. */
-
-    read_str(&v->iconlabel, _("calculator"));  /* Default icon label. */
-
-    init_constant(0, "0.621");                 /* kms/hr <=> miles/hr. */
-    init_constant(1, "1.4142135623");          /* square root of 2 */
-    init_constant(2, "2.7182818284");          /* e */
-    init_constant(3, "3.1415926535");          /* pi */
-    init_constant(4, "0.3937007");             /* cms <=> inch. */
-    init_constant(5, "57.295779513");          /* degrees/radian. */
-    init_constant(6, "1048576.0");             /* 2 ^ 20. */
-    init_constant(7, "0.0353");                /* grams <=> ounce. */
-    init_constant(8, "0.948");                 /* Kjoules <=> BTU's. */
-    init_constant(9, "0.0610");                /* cms3 <=> inches3. */
-
-    n = 0;
-    for (i = 0; i < MAXREGS; i++) {
-        mpcim(&n, v->MPmvals[i]);
-    }
-}
-
-
-void
-read_cfdefs()        /* Read constant/function definitions. */
-{
-    int n;
-
-    for (n = 0; n < MAXCONFUN; n++) {
-        get_constant(n);
-        STRCPY(v->fun_vals[n], "");    /* Initially empty function strings. */
-        get_function(n);
-    }
-}
-
-
 void
 read_resources()    /* Read all possible resources from the database. */
 {
     int boolval, i, intval;
-    char str[MAXLINE];
+    char key[MAXLINE], str[MAXLINE];
 
     if (get_int_resource(R_ACCURACY, &intval)) {
         v->accuracy = intval;
@@ -314,6 +241,13 @@ read_resources()    /* Read all possible resources from the database. */
         }
     }
 
+    for (i = 0; i < MAX_REGISTERS; i++) {
+        SNPRINTF(key, MAXLINE, "register%d", i);
+        if (get_str_resource(key, str)) {
+            MPstr_to_num(str, DEC, v->MPmvals[i]);
+        }
+    }
+
     if (get_str_resource(R_BASE, str)) {
         for (i = 0; i < MAXBASES; i++) {
             if (EQUAL(str, Rbstr[i])) {
@@ -374,35 +308,47 @@ read_resources()    /* Read all possible resources from the database. */
     }
 
     /* Set expression (arithmetic precedence) mode as default. */
-    v->syntax = exprs;
+    v->syntax = EXPRS;
 
     if (get_str_resource(R_SYNTAX, str)) {
-      for (i = 0; i < MAXSYNTAX; i++) {
-	if (EQUAL(str, Rsstr[i])) {
-	  break;
-	}
-      }
+        for (i = 0; i < MAXSYNTAX; i++) {
+            if (EQUAL(str, Rsstr[i])) {
+                break;
+            }
+        }
+
+        if (i == MAXSYNTAX) {
+            FPRINTF(stderr, _("%s: invalid syntax mode [%s]\n"), 
+                    v->progname, str);
+        } else {
+            v->syntax = i;
+        }
+    }
+
+    v->bitcalculating_mode = 0;            /* default = off. */
+
+    if (get_str_resource(R_BITCALC, str)) {
+        for (i = 0; i < MAXBITCALC; i++) {
+            if (EQUAL(str, Rcstr[i])) {
+                break;
+            }
+        }
       
-      if (i == MAXSYNTAX) {
-	FPRINTF(stderr, _("%s: invalid syntax mode [%s]\n"), 
-		v->progname, str);
-      } else {
-	v->syntax = i;
-      }
+        if (i == MAXBITCALC) {
+            FPRINTF(stderr, _("%s: invalid bitcalc choice [%s]\n"), 
+                    v->progname, str);
+        } else {
+            v->bitcalculating_mode = i;
+        }
     }
 
-    if (get_bool_resource(R_REGS, &boolval)) {
-        v->rstate = boolval;
-    }
-
-    if (get_bool_resource(R_ZEROES, &boolval)) {
+    if (get_boolean_resource(R_ZEROES, &boolval)) {
         v->show_zeroes = boolval;
     }
 
-    if (get_bool_resource(R_TSEP, &boolval)) {
+    if (get_boolean_resource(R_TSEP, &boolval)) {
         v->show_tsep = boolval;
     }
-
 }
 
 
@@ -421,18 +367,13 @@ read_str(char **str, char *value)
 }
 
 
-char *
-set_bool(int value)
-{
-    return(value ? "true" : "false");
-}
-
-
 void
-usage(char *progname)
-{
-    FPRINTF(stderr, _("%s version %s\n\n"), progname, VERSION);
-    FPRINTF(stderr, _("Usage: %s: [-D] [-E] [-a accuracy] "), progname);
-    FPRINTF(stderr, _("\t\t [-?] [-v] [-?]\n"));
-    exit(1);
+resources_init()        /* Load gconf configuration database for gcalctool. */
+{ 
+    char str[MAXLINE];
+
+    assert(client == NULL);
+    SNPRINTF(str, MAXLINE, "/apps/%s", v->appname);
+    client = gconf_client_get_default();
+    gconf_client_add_dir(client, str, GCONF_CLIENT_PRELOAD_NONE, NULL);
 }
