@@ -1,7 +1,7 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/mpmath.h,v 1.12 2006/09/13 17:41:46 sampie Exp $
+/*  $Header$
  *
- *  Copyright (c) 1987-2006 Sun Microsystems, Inc. All Rights Reserved.
+ *  Copyright (c) 1987-2007 Sun Microsystems, Inc. All Rights Reserved.
  *           
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,6 +22,10 @@
 #ifndef MPMATH_H
 #define MPMATH_H
 
+#include "mp.h"
+
+typedef unsigned long  BOOLEAN;
+
 /* function parameters: t=target, s=source) */
 
 BOOLEAN ibool(double x);
@@ -59,7 +63,7 @@ void calc_rate(int t[MP_SIZE]);
 void calc_sln(int t[MP_SIZE]);
 void calc_syd(int t[MP_SIZE]);
 void calc_term(int t[MP_SIZE]);
-void calc_rshift(int s[MP_SIZE], int t[MP_SIZE], int times, enum shiftd dir);
+void calc_shift(int s[MP_SIZE], int t[MP_SIZE], int times);
 void calc_epowy(int s[MP_SIZE], int t[MP_SIZE]);
 
 /* return true if parameter is integer */
